///////////////////////////////////////////////////////////////////////////////
//                            X3DComponentVisitor.inl                        //
///////////////////////////////////////////////////////////////////////////////

namespace X3DTK {

template<class T> 
void X3DComponentVisitor::defineEnterFunction(void (*ptrF)(T *))
{
  SFType::beginFunctionDefinition();
  EnterFunction *EF = new EnterFunction(reinterpret_cast<ptrToEnterFunction>(ptrF));
  //finding if an occurence of T
  T *t = new T();
  SFString name = t->getTypeName();
  SFComponent *TComponent = t->getType()->getComponent();
  //checking if the node belongs to the component.
  if (TComponent != component)
  {
    SFString sg1, sg2;
    if (component->getSceneGraphName() != "")
      sg1 = " of the " + component->getSceneGraphName() + " scene graph";
    if (t->getType()->getSceneGraphName() != "")
      sg1 = " of the " + t->getType()->getSceneGraphName() + " scene graph";  

    cx3d << "warning: defineEnterFunction for " << name << " which doesn't belong to the " << component->getName() << " component" << sg1 << "," << std::endl;
    cx3d << "         but to the " << t->getType()->getComponentName() << " component" << sg2 << std::endl;
  }
  else
    _enterDict[t->getType()->getEncodedName()] = EF;

  delete t;
  SFType::endFunctionDefinition();
}
  
template<class T> 
void X3DComponentVisitor::defineWalkOnFunction(bool (*ptrF)(T *, SFAbstractNode))
{
  SFType::beginFunctionDefinition();
  WalkOnFunction *WF = new WalkOnFunction(reinterpret_cast<ptrToWalkOnFunction>(ptrF));
  //finding if an occurence of T
  T *t = new T();
  SFString name = t->getTypeName();
  SFComponent *TComponent = t->getType()->getComponent();
  //checking if the node belongs to the component.
  if (TComponent != component)
  {
    SFString sg1, sg2;
    if (component->getSceneGraphName() != "")
      sg1 = " of the " + component->getSceneGraphName() + " scene graph";
    if (t->getType()->getSceneGraphName() != "")
      sg1 = " of the " + t->getType()->getSceneGraphName() + " scene graph";  

    cx3d << "warning: defineWalkOnFunction for " << name << " which doesn't belong to the " << component->getName() << " component" << sg1 << "," << std::endl;
    cx3d << "         but to the " << t->getType()->getComponentName() << " component" << sg2 << std::endl;    
  }
  else
    _walkonDict[t->getType()->getEncodedName()] = WF;
    
  delete t;
  SFType::endFunctionDefinition();
};

template<class T> 
void X3DComponentVisitor::defineLeaveFunction(void (*ptrF)(T *))
{
  SFType::beginFunctionDefinition();
  LeaveFunction *LF = new LeaveFunction(reinterpret_cast<ptrToLeaveFunction>(ptrF));
  //finding if an occurence of T
  T *t = new T();
  SFString name = t->getTypeName();
  SFComponent *TComponent = t->getType()->getComponent();
  //checking if the node belongs to the component.
  if (TComponent != component)
  {
    SFString sg1, sg2;
    if (component->getSceneGraphName() != "")
      sg1 = " of the " + component->getSceneGraphName() + " scene graph";
    if (t->getType()->getSceneGraphName() != "")
      sg1 = " of the " + t->getType()->getSceneGraphName() + " scene graph";  

    cx3d << "warning: defineLeaveFunction for " << name << " which doesn't belong to the " << component->getName() << " component" << sg1 << "," << std::endl;
    cx3d << "         but to the " << t->getType()->getComponentName() << " component" << sg2 << std::endl;    
  }
  else
    _leaveDict[t->getType()->getEncodedName()] = LF;
    
  delete t;
  SFType::endFunctionDefinition();
};

}  
