namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
TransformComputerCoreVisitor<MData, VData, EData, FData, readOnly>::TransformComputerCoreVisitor()
{
  // Enter functions.
  defineEnterFunction(&TransformComputerCoreVisitor::enterMesh);
  defineEnterFunction(&TransformComputerCoreVisitor::enterTransform);
  // Leave function
  defineLeaveFunction(&TransformComputerCoreVisitor::leaveX3DGroupingNode);
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void TransformComputerCoreVisitor<MData, VData, EData, FData, readOnly>::enterMesh(Mesh<MData, VData, EData, FData, readOnly> *M)
{
  // StateVariables assignation.
  TransformComputerStateVariables<MData, VData, EData, FData, readOnly> *stateVariables = GraphTraversal::getInstanceOf<TransformComputerStateVariables<MData, VData, EData, FData, readOnly> >();
  
  std::cout << "enter Mesh" << std::endl;
  std::cout << "Transform:" << std::endl << stateVariables->getMatrix() << std::endl;
  std::cout << "  number of vertices = " << M->getVertices().size() << std::endl;
  
  // Test the optional presence of MESH::VertexPointData in VData.
  if (VData::template find<MESH::VertexPointData>())
  {
    // Accessing the MESH::VertexPointData of VData.
    for (typename Mesh<MData, VData, EData, FData, readOnly>::MFVertex::const_iterator it = M->getVertices().begin(); it != M->getVertices().end(); ++it)
      std::cout << (*it)->template ogetData<MESH::VertexPointData>().getPoint() << std::endl;
  }    
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void TransformComputerCoreVisitor<MData, VData, EData, FData, readOnly>::enterTransform(Transform *T)
{
  // Pushing the current transformation matrix on the stack.
  GraphTraversal::getInstanceOf<TransformComputerStateVariables<MData, VData, EData, FData, readOnly> >()->pushMatrix(T->getTransform());
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void TransformComputerCoreVisitor<MData, VData, EData, FData, readOnly>::leaveX3DGroupingNode(X3DGroupingNode *)
{
  // Poping the current transformation matrix of the stack.
  GraphTraversal::getInstanceOf<TransformComputerStateVariables<MData, VData, EData, FData, readOnly> >()->popMatrix();
}

}
}
